@@ -3,8 +3,8 @@ __author__ = ", ".join(["Shyue Ping Ong", "Anubhav Jain", "Geoffroy Hautier",
                         "Sai Jayaraman", "Michael Kocher", "Dan Gunter",
                         "Shreyas Cholia", "Vincent L Chevrier",
                         "Rickard Armiento"])
-__date__ = "Sep 30 2013"
-__version__ = "2.8.7"
+__date__ = "Dec 18 2013"
+__version__ = "2.8.10"
 
 #Useful aliases for commonly used objects and modules.
 
