@@ -989,7 +989,7 @@ class IStructure(SiteCollection, MSONable):
         """
         Compute the scalar producr of vector(s) either in real space or
         reciprocal space.
-                                                                                                
+
         Args:
             coords:
                 Array-like object with the coordinates.
@@ -1644,7 +1644,7 @@ class Structure(IStructure):
             new_atom_occu = collections.defaultdict(int)
             for sp, amt in site.species_and_occu.items():
                 if sp in species_mapping:
-                    if isinstance(species_mapping[sp], dict):
+                    if isinstance(species_mapping[sp], collections.Mapping):
                         for new_sp, new_amt in species_mapping[sp].items():
                             new_atom_occu[smart_element_or_specie(new_sp)] \
                                 += amt * new_amt
@@ -1677,11 +1677,11 @@ class Structure(IStructure):
 
     def remove_species(self, species):
         """
-        Remove all occurrences of a species from a structure.
+        Remove all occurrences of several species from a structure.
 
         Args:
             species:
-                species to remove.
+                Sequence of species to remove, e.g., ["Li", "Na"].
         """
         new_sites = []
         species = map(smart_element_or_specie, species)
@@ -2137,7 +2137,7 @@ class Molecule(IMolecule):
                             new_atom_occu[species_mapping[sp]] += amt
                         else:
                             new_atom_occu[species_mapping[sp]] = amt
-                    elif isinstance(species_mapping[sp], dict):
+                    elif isinstance(species_mapping[sp], collections.Mapping):
                         for new_sp, new_amt in species_mapping[sp].items():
                             if new_sp in new_atom_occu:
                                 new_atom_occu[new_sp] += amt * new_amt
