@@ -36,6 +36,20 @@ class Correction(object):
     __metaclass__ = abc.ABCMeta
 
     @abc.abstractmethod
+    def get_correction(self, entry):
+        """
+        Returns correction for a single entry.
+
+        Args:
+            entry:
+                A ComputedEntry object.
+
+        Returns:
+            The energy correction ot be applied. None if the entry is
+            incompatible.
+        """
+        return
+
     def correct_entry(self, entry):
         """
         Corrects a single entry.
@@ -48,7 +62,11 @@ class Correction(object):
             An processed entry. None if entry is not compatible within the
             processing scheme.
         """
-        return
+        c = self.get_correction(entry)
+        if c is not None:
+            entry.correction += c
+            return entry
+        return None
 
 
 class PotcarCorrection(Correction):
@@ -83,19 +101,23 @@ class PotcarCorrection(Correction):
             raise ValueError("Only MIT and MP POTCAR corrections are "
                              "supported currently.")
         self.valid_potcars = set(input_set.potcar_settings.values())
+        self.name = name
 
-    def correct_entry(self, entry):
+    def get_correction(self, entry):
         try:
             psp_settings = set([sym.split(" ")[1]
                                 for sym
                                 in entry.parameters["potcar_symbols"]])
         except KeyError:
-            raise ValueError("PotcarCorrection can only be "
-                             "checked for entries with a \"potcar_symbols\" in "
-                             "entry.parameters")
+            raise ValueError(
+                "PotcarCorrection can only be checked for entries with a "
+                "\"potcar_symbols\" in entry.parameters")
         if not self.valid_potcars.issuperset(psp_settings):
             return None
-        return entry
+        return 0
+
+    def __str__(self):
+        return "{} Potcar Correction".format(self.name)
 
 
 class GasCorrection(Correction):
@@ -121,14 +143,15 @@ class GasCorrection(Correction):
         self.oxide_correction = {
             k: float(v) for k, v
             in config.items("{}OxideCorrection".format(name))}
+        self.name = name
 
-    def correct_entry(self, entry):
+    def get_correction(self, entry):
         comp = entry.composition
+
         rform = entry.composition.reduced_formula
         if rform in self.cpd_energies:
-            entry.correction += self.cpd_energies[rform] * comp.num_atoms \
+            return self.cpd_energies[rform] * comp.num_atoms \
                 - entry.uncorrected_energy
-            return entry
 
         correction = 0
         #Check for oxide, peroxide, superoxide, and ozonide corrections.
@@ -138,12 +161,18 @@ class GasCorrection(Correction):
                     ox_corr = self.oxide_correction[
                         entry.data["oxide_type"]]
                     correction += ox_corr * comp["O"]
+                if entry.data["oxide_type"] == "hydroxide":
+                    ox_corr = self.oxide_correction["oxide"]
+                    correction += ox_corr * comp["O"]
+
             elif hasattr(entry, "structure"):
                 ox_type, nbonds = oxide_type(entry.structure, 1.05,
                                              return_nbonds=True)
                 if ox_type in self.oxide_correction:
                     correction += self.oxide_correction[ox_type] * \
                         nbonds
+                elif ox_type == "hydroxide":
+                    correction += self.oxide_correction["oxide"] * comp["O"]
             else:
                 if rform in UCorrection.common_peroxides:
                     correction += self.oxide_correction["peroxide"] * \
@@ -157,8 +186,10 @@ class GasCorrection(Correction):
                 elif Element("O") in comp.elements and len(comp.elements) > 1:
                     correction += self.oxide_correction['oxide'] * comp["O"]
 
-        entry.correction += correction
-        return entry
+        return correction
+
+    def __str__(self):
+        return "{} Gas Correction".format(self.name)
 
 
 class AqueousCorrection(Correction):
@@ -178,25 +209,28 @@ class AqueousCorrection(Correction):
         config.optionxform = str
         config.readfp(open(os.path.join(module_dir,
                                         "Compatibility.cfg")))
-
         cpd_energies = dict(
             config.items("{}AqueousCompoundEnergies".format(name)))
-
         self.cpd_energies = {k: float(v) for k, v in cpd_energies.items()}
+        self.name = name
 
-    def correct_entry(self, entry):
+    def get_correction(self, entry):
         comp = entry.composition
         rform = comp.reduced_formula
         cpdenergies = self.cpd_energies
+        correction = 0
         if rform in cpdenergies:
             if rform in ["H2", "H2O"]:
-                entry.correction = cpdenergies[rform] * comp.num_atoms \
-                    - entry.uncorrected_energy
+                correction = cpdenergies[rform] * comp.num_atoms \
+                    - entry.uncorrected_energy - entry.correction
             else:
-                entry.correction += cpdenergies[rform] * comp.num_atoms
+                correction += cpdenergies[rform] * comp.num_atoms
         if not rform == "H2O":
-            entry.correction += 0.5 * 2.46 * min(comp["H"]/2.0, comp["O"])
-        return entry
+            correction += 0.5 * 2.46 * min(comp["H"]/2.0, comp["O"])
+        return correction
+
+    def __str__(self):
+        return "{} Aqueous Correction".format(self.name)
 
 
 class UCorrection(Correction):
@@ -259,7 +293,10 @@ class UCorrection(Correction):
             self.u_corrections = {}
             self.u_settings = {}
 
-    def correct_entry(self, entry):
+        self.name = name
+        self.compat_type = compat_type
+
+    def get_correction(self, entry):
         if entry.parameters.get("run_type", "GGA") == "HF":
             return None
 
@@ -283,8 +320,10 @@ class UCorrection(Correction):
             if sym in ucorr:
                 correction += float(ucorr[sym]) * comp[el]
 
-        entry.correction += correction
-        return entry
+        return correction
+
+    def __str__(self):
+        return "{} {} Correction".format(self.name, self.compat_type)
 
 
 class Compatibility(object):
@@ -317,13 +356,22 @@ class Compatibility(object):
             An adjusted entry if entry is compatible, otherwise None is
             returned.
         """
-        entry.correction = 0
-        for c in self.corrections:
-            entry = c.correct_entry(entry)
-            if entry is None:
-                return None
+        corrections = self.get_corrections_dict(entry)
+        if corrections is None:
+            return None
+        entry.correction = sum(corrections.values())
         return entry
 
+    def get_corrections_dict(self, entry):
+        corrections = {}
+        for c in self.corrections:
+            val = c.get_correction(entry)
+            if val is None:
+                return None
+            elif val != 0:
+                corrections[str(c)] = val
+        return corrections
+
     def process_entries(self, entries):
         """
         Process a sequence of entries with the chosen Compatibility scheme.
