@@ -1,7 +1,7 @@
-from .calculations import *
 from .eos import *
 from .pseudos import *
 from .netcdf import *
 from .events import *
-from .task import *
-from .workflow import *
+from .tasks import *
+from .workflows import *
+from .calculations import *
