@@ -7,7 +7,7 @@ import os
 import sys
 import time
 import collections
-import itertools
+import warnings
 import cPickle as pickle
 #import pickle as pickle
 
@@ -17,7 +17,7 @@ except ImportError:
     pass
 
 from pymatgen.util.io_utils import FileLock
-from pymatgen.util.string_utils import pprint_table
+from pymatgen.util.string_utils import pprint_table, is_string
 from pymatgen.io.abinitio.tasks import Dependency, Node, Task, ScfTask, PhononTask 
 from pymatgen.io.abinitio.utils import Directory
 from pymatgen.io.abinitio.abiinspect import yaml_read_irred_perts
@@ -41,7 +41,6 @@ __all__ = [
 
 
 class AbinitFlow(Node):
-#class AbinitFlow(collections.Iterable):
     """
     This object is a container of workflows. Its main task is managing the 
     possible inter-depedencies among the workflows and the creation of
@@ -59,26 +58,24 @@ class AbinitFlow(Node):
 
     PICKLE_FNAME = "__AbinitFlow__.pickle"
 
-    def __init__(self, workdir, manager, auto_restart=False, pickle_protocol=-1):
+    def __init__(self, workdir, manager, pickle_protocol=-1):
         """
         Args:
             workdir:
                 String specifying the directory where the workflows will be produced.
             manager:
                 `TaskManager` object responsible for the submission of the jobs.
-            auto_restart:
-                True if unconverged calculations should be restarted automatically.
             pickle_procol:
                 Pickle protocol version used for saving the status of the object.
                 -1 denotes the latest version supported by the python interpreter.
         """
         super(AbinitFlow, self).__init__()
 
-        self.workdir = os.path.abspath(workdir)
+        self.set_workdir(workdir)
+
         self.creation_date = time.asctime()
 
         self.manager = manager.deepcopy()
-        self.auto_restart = auto_restart
 
         # List of workflows.
         self._works = []
@@ -86,13 +83,9 @@ class AbinitFlow(Node):
         # List of callbacks that must be executed when the dependencies reach S_OK
         self._callbacks = []
 
-        # Directories with (input|output|temporary) data.
-        self.indir = Directory(os.path.join(self.workdir, "indata"))
-        self.outdir = Directory(os.path.join(self.workdir, "outdata"))
-        self.tmpdir = Directory(os.path.join(self.workdir, "tmpdata"))
-
         self.pickle_protocol = int(pickle_protocol)
 
+        # TODO
         # Signal slots: a dictionary with the list 
         # of callbacks indexed by node_id and SIGNAL_TYPE.
         # When the node changes its status, it broadcast a signal.
@@ -105,6 +98,62 @@ class AbinitFlow(Node):
         #for task in self.iflat_tasks():
         #    slots[task] = {s: [] for s in work.S_ALL}
 
+    def set_workdir(self, workdir, chroot=False):
+        """Set the working directory. Cannot be set more than once unless chroot is True"""
+        if not chroot and hasattr(self, "workdir") and self.workdir != workdir:
+            raise ValueError("self.workdir != workdir: %s, %s" % (self.workdir,  workdir))
+
+        # Directories with (input|output|temporary) data.
+        self.workdir = os.path.abspath(workdir)
+        self.indir = Directory(os.path.join(self.workdir, "indata"))
+        self.outdir = Directory(os.path.join(self.workdir, "outdata"))
+        self.tmpdir = Directory(os.path.join(self.workdir, "tmpdata"))
+
+    @classmethod
+    def pickle_load(cls, filepath, disable_signals=False):
+        """
+        Loads the object from a pickle file and performs initial setup.
+
+        Args:
+            filepath:
+                Filename or directory name. It filepath is a directory, we 
+                scan the directory tree starting from filepath and we 
+                read the first pickle database.
+            disable_signals:
+                If True, the nodes of the flow are not connected by signals.
+                This option is usually used when we want to read a flow 
+                in read-only mode and we want to avoid any possible side effect.
+        """
+        if os.path.isdir(filepath):
+            # Walk through each directory inside path and find the pickle database.
+            for dirpath, dirnames, filenames in os.walk(filepath):
+                fnames = [f for f in filenames if f == cls.PICKLE_FNAME]
+                if fnames:
+                    assert len(fnames) == 1
+                    filepath = os.path.join(dirpath, fnames[0])
+                    break
+            else:
+                err_msg = "Cannot find %s inside directory %s" % (cls.PICKLE_FNAME, filepath)
+                raise ValueError(err_msg)
+
+        with FileLock(filepath) as lock:
+            with open(filepath, "rb") as fh:
+                flow = pickle.load(fh)
+
+        # Check if versions match.
+        if flow.VERSION != cls.VERSION:
+            msg = ("File flow version %s != latest version %s\n."
+                   "Regerate the flow to solve the problem " % (flow.VERSION, cls.VERSION))
+            warnings.warn(msg)
+
+        if not disable_signals:
+            flow.connect_signals()
+
+        # Recompute the status of each task since tasks that
+        # have been submitted previously might be completed.
+        flow.check_status()
+        return flow
+
     def __len__(self):
         return len(self.works)
 
@@ -124,22 +173,129 @@ class AbinitFlow(Node):
         """True if all the tasks in workflows have reached S_OK."""
         return all(work.all_ok for work in self)
 
-    #@property
-    #def completed(self):
-    #    """True if all the tasks of the flow have reached S_OK."""
-    #    return all(task.status == task.S_OK for task in self.iflat_tasks())
+    @property
+    def all_tasks(self):
+        return self.iflat_tasks()
+
+    @property
+    def num_tasks(self):
+        """Total number of tasks"""
+        return len(list(self.iflat_tasks()))
+
+    @property
+    def errored_tasks(self):
+        return self.iflat_tasks(status=self.S_ERROR)
+
+    @property
+    def num_errored_tasks(self):
+        """The number of tasks whose status is `S_ERROR`."""
+        return len(list(self.errored_tasks))
+
+    @property
+    def unconverged_tasks(self):
+        return self.iflat_tasks(status=self.S_UNCONVERGED)
+
+    @property
+    def num_unconverged_tasks(self):
+        """The number of tasks whose status is `S_UNCONVERGED`."""
+        return len(list(self.unconverged_tasks))
+
+    @property
+    def status_counter(self):
+        """
+        Returns a `Counter` object that counts the number of task with 
+        given status (use the string representation of the status as key).
+        """
+        # Count the number of tasks with given status in each workflow.
+        counter = self[0].status_counter
+        for work in self[1:]:
+            counter += work.status_counter
+
+        return counter
+
+    @property
+    def ncpus_reserved(self):
+        """
+        Returns the number of CPUs reserved in this moment.
+        A CPUS is reserved if it's still not running but 
+        we have submitted the task to the queue manager.
+        """
+        return sum(work.ncpus_reverved for work in self)
+
+    @property
+    def ncpus_allocated(self):
+        """
+        Returns the number of CPUs allocated in this moment.
+        A CPU is allocated if it's running a task or if we have
+        submitted a task to the queue manager but the job is still pending.
+        """
+        return sum(work.ncpus_allocated for work in self)
+
+    @property
+    def ncpus_inuse(self):
+        """
+        Returns the number of CPUs used in this moment.
+        A CPU is used if there's a job that is running on it.
+        """
+        return sum(work.ncpus_inuse for work in self)
+
+    @property
+    def has_chrooted(self):
+        """
+        Returns a string that evaluates to True if we have changed 
+        the workdir for visualization purposes e.g. we are using sshfs.
+        The string gives the previous workdir of the flow.
+        """
+        try:
+            return self._chrooted_from
+
+        except AttributeError:
+            return ""
+
+    def chroot(self, new_workdir):
+        self._chrooted_from = self.workdir
+        self.set_workdir(new_workdir, chroot=True)
+
+        for i, work in enumerate(self):
+            new_wdir = os.path.join(self.workdir, "work_" + str(i))
+            work.chroot(new_wdir)
+
+    def groupby_status(self):
+        """
+        Returns a ordered dictionary mapping the task status to 
+        the list of named tuples (task, work_index, task_index).
+        """
+        Entry = collections.namedtuple("Entry", "task wi ti")
+        d = collections.defaultdict(list)
+
+        for task, wi, ti in self.iflat_tasks_wti():
+            d[task.status].append(Entry(task, wi, ti))
+
+        # Sort keys according to their status.
+        return collections.OrderedDict([(k, d[k]) for k in sorted(list(d.keys()))])
 
     def iflat_tasks_wti(self, status=None, op="="):
         """
-        Returns:
+        Generator to iterate over all the tasks of the `Flow`.
+        Yields
+
             (task, work_index, task_index)
+
+        If status is not None, only the tasks whose status satisfies
+        the condition (task.status op status) are selected
+        status can be either one of the flags defined in the `Task` class 
+        (e.g Task.S_OK) or a string e.g "S_OK" 
         """
         return self._iflat_tasks_wti(status=status, op=op, with_wti=True)
 
     def iflat_tasks(self, status=None, op="="):
         """
-        Returns:
-            task
+        Generator to iterate over all the tasks of the `Flow`.
+
+        If status is not None, only the tasks whose status satisfies
+        the condition (task.status op status) are selected
+        status can be either one of the flags defined in the `Task` class 
+        (e.g Task.S_OK) or a string e.g "S_OK" 
         """
         return self._iflat_tasks_wti(status=status, op=op, with_wti=False)
 
@@ -171,6 +327,10 @@ class AbinitFlow(Node):
                 "<=": operator.le,
             }[op]
 
+            # Accept Task.S_FLAG or string.
+            if is_string(status):
+                status = getattr(Task, status)
+
             for wi, work in enumerate(self):
                 for ti, task in enumerate(work):
                     if op(task.status, status):
@@ -179,52 +339,37 @@ class AbinitFlow(Node):
                         else:
                             yield task
 
-    @property
-    def ncpus_reserved(self):
-        """
-        Returns the number of CPUs reserved in this moment.
-        A CPUS is reserved if it's still not running but 
-        we have submitted the task to the queue manager.
-        """
-        return sum(work.ncpus_reverved for work in self)
+    def check_dependencies(self):
+        """Test the dependencies of the nodes for possible deadlocks."""
+        deadlocks = []
 
-    @property
-    def ncpus_allocated(self):
-        """
-        Returns the number of CPUs allocated in this moment.
-        A CPU is allocated if it's running a task or if we have
-        submitted a task to the queue manager but the job is still pending.
-        """
-        return sum(work.ncpus_allocated for work in self)
+        for task in self.all_tasks:
+            for dep in task.deps:
+                if dep.node.depends_on(task):
+                    deadlocks.append((task, dep.node))
 
-    @property
-    def ncpus_inuse(self):
-        """
-        Returns the number of CPUs used in this moment.
-        A CPU is used if there's a job that is running on it.
-        """
-        return sum(work.ncpus_inuse for work in self)
+        if deadlocks:
+           lines = ["Detect wrong list of dependecies that will lead to a deadlock:"]
+           lines.extend(["%s <--> %s" % nodes for nodes in deadlocks])
+           raise ValueError("\n".join(lines))
+
+    #def detect_deadlock(self):
+    #    eu_tasks = list(self.errored_tasks) + list(self.unconverged_tasks)
+    #     if not eu_tasks:
+    #        return []
+
+    #    deadlocked = []
+    #    for task in self.all_tasks:
+    #        if any(task.depends_on(eu_task) for eu_task in eu_tasks):
+    #            deadlocked.append(task)
+
+    #    return deadlocked
 
     def check_status(self):
         """Check the status of the workflows in self."""
         for work in self:
             work.check_status()
 
-        # Test whether some task should be restarted.
-        if self.auto_restart:
-            num_restarts = 0
-            for task, wt in self.iflat_tasks(status=Task.S_UNCONVERGED):
-                msg = "AbinitFlow will try restart task %s" % task
-                print(msg)
-                logger.info(msg)
-                retcode = task.restart_if_needed()
-                if retcode == 0: 
-                    num_restarts += 1
-
-            if num_restarts:
-                print("num_restarts done successfully: ", num_restarts)
-                self.pickle_dump()
-
     def show_status(self, stream=sys.stdout):
         """
         Report the status of the workflows and the status 
@@ -234,11 +379,10 @@ class AbinitFlow(Node):
             print(80*"=")
             print("Workflow #%d: %s, Finalized=%s\n" % (i, work, work.finalized) )
 
-            table = [[
-                     "Task", "Status", "Queue_id", 
-                     "Errors", "Warnings", "Comments", 
-                     "MPI", "OMP", 
-                     "num_restarts", "max_restarts", "Task Class"
+            table = [["Task", "Status", "Queue_id", 
+                      "Errors", "Warnings", "Comments", 
+                      "MPI", "OMP", 
+                      "num_restarts", "Task Class"
                      ]]
 
             for task in work:
@@ -252,7 +396,7 @@ class AbinitFlow(Node):
                     events = map(str, [report.num_errors, report.num_warnings, report.num_comments])
 
                 cpu_info = map(str, [task.mpi_ncpus, task.omp_ncpus])
-                task_info = map(str, [task.num_restarts, task.max_num_restarts, task.__class__.__name__])
+                task_info = map(str, [task.num_restarts, task.__class__.__name__])
 
                 table.append(
                     [task_name, str(task.status), str(task.queue_id)] + 
@@ -263,6 +407,29 @@ class AbinitFlow(Node):
 
             pprint_table(table, out=stream)
 
+    def cancel(self):
+        """
+        Cancel all the tasks that are in the queue.
+
+        Returns:
+            Number of jobs cancelles.
+        """
+        # If we are running with the scheduler, we must send a SIGKILL signal.
+        pid_file = os.path.join(self.workdir, "_PyFlowScheduler.pid")
+        if os.path.exists(pid_file):
+            with open(pid_file, "r") as fh:
+                pid = int(fh.readline())
+                
+            retcode = os.system("kill -9 %d" % pid)
+            print("Sent SIGKILL to the scheduler, retcode = %s" % retcode)
+            #os.remove(pid_file)
+
+        num_cancelled = 0
+        for task in self.iflat_tasks():
+            num_cancelled += task.cancel()
+
+        return num_cancelled
+
     def build(self, *args, **kwargs):
         """Make directories and files of the `Flow`."""
         self.indir.makedirs()
@@ -289,6 +456,10 @@ class AbinitFlow(Node):
         Returns:
             0 if success
         """
+        if self.has_chrooted:
+            print("cannot pickle_dump since we have chrooted from %s" % self.has_chrooted)
+            return -1
+
         protocol = self.pickle_protocol
         filepath = os.path.join(self.workdir, self.PICKLE_FNAME)
 
@@ -319,45 +490,6 @@ class AbinitFlow(Node):
         #        pass
         return 0
 
-    @classmethod
-    def pickle_load(cls, filepath, disable_signals=False):
-        """
-        Loads the object from a pickle file and performs initial setup.
-
-        Args:
-            filepath:
-                Filename or directory name. It filepath is a directory, we 
-                scan the directory tree starting from filepath and we 
-                read the first pickle database.
-            disable_signals:
-                If True, the nodes of the flow are not connected by signals.
-                This option is usually used when we want to read a flow 
-                in read-only mode and we want to avoid any possible side effect.
-        """
-        if os.path.isdir(filepath):
-            # Walk through each directory inside path and find the pickle database.
-            for dirpath, dirnames, filenames in os.walk(filepath):
-                fnames = [f for f in filenames if f == cls.PICKLE_FNAME]
-                if fnames:
-                    assert len(fnames) == 1
-                    filepath = os.path.join(dirpath, fnames[0])
-                    break
-            else:
-                err_msg = "Cannot find %s inside directory %s" % (cls.PICKLE_FNAME, path)
-                raise ValueError(err_msg)
-
-        with FileLock(filepath) as lock:
-            with open(filepath, "rb") as fh:
-                flow = pickle.load(fh)
-
-        if not disable_signals:
-            flow.connect_signals()
-
-        # Recompute the status of each task since tasks that
-        # have been submitted previously might be completed.
-        flow.check_status()
-        return flow
-
     def register_task(self, input, deps=None, manager=None, task_class=None):
         """
         Utility function that generates a `Workflow` made of a single task
@@ -407,7 +539,7 @@ class AbinitFlow(Node):
         if workdir is None:
             work_workdir = os.path.join(self.workdir, "work_" + str(len(self)))
         else:
-            work_workdir = os.path.join(self.workdir, os.path.basenane(workdir))
+            work_workdir = os.path.join(self.workdir, os.path.basename(workdir))
 
         work.set_workdir(work_workdir)
 
@@ -477,6 +609,8 @@ class AbinitFlow(Node):
         for task in self.iflat_tasks():
             task.set_flow(self)
 
+        self.check_dependencies()
+
         return self
 
     def show_dependencies(self):
@@ -698,7 +832,7 @@ def phonon_flow(workdir, manager, scf_input, ph_inputs):
     # Build a temporary workflow with a shell manager just to run 
     # ABINIT to get the list of irreducible pertubations for this q-point.
     shell_manager = manager.to_shell_manager(mpi_ncpus=1)
-    print(shell_manager)
+    #print(shell_manager)
 
     if not isinstance(ph_inputs, (list, tuple)):
         ph_inputs = [ph_inputs]
@@ -726,6 +860,8 @@ def phonon_flow(workdir, manager, scf_input, ph_inputs):
 
         # Parse the file to get the perturbations.
         irred_perts = yaml_read_irred_perts(fake_task.log_file.path)
+        #import sys
+        #sys.exit(1)
         print(irred_perts)
         w.rmtree()
 
