@@ -10,11 +10,9 @@ import abc
 import collections
 import json
 import warnings
-import cPickle as pickle
-import cStringIO as StringIO
 import numpy as np
 
-from pymatgen.core.design_patterns import FrozenDict
+from pymatgen.core.design_patterns import FrozenDict, AttrDict
 from pymatgen.core.periodic_table import PeriodicTable
 from pymatgen.util.num_utils import iterator_from_slice
 from pymatgen.util.string_utils import list_strings, is_string
@@ -68,7 +66,7 @@ def l2str(l):
     try:
         return _l2str[l]
     except KeyError:
-        return "Unknown: received l = %s" % l
+        return "Unknown angular momentum, received l = %s" % l
 
 
 def str2l(s):
@@ -111,6 +109,10 @@ class Pseudo(object):
     """
     __metaclass__ = abc.ABCMeta
 
+    #def __init__(self, filepath):
+    #    self.filepath = os.path.abspath(filepath)
+    #    self._dojo_report = {}
+
     @classmethod
     def aspseudo(cls, obj):
         """
@@ -144,7 +146,8 @@ class Pseudo(object):
         app("<%s: %s>" % (self.__class__.__name__, self.name))
         app("  summary: " + self.summary.strip())
         app("  number of valence electrons: %s" % self.Z_val)
-        app("  XC correlation (ixc): %s" % self._pspxc)  #FIXME
+        #FIXME: rewrite the treatment of xc, use XML specs as starting point
+        #app("  XC correlation (ixc): %s" % self._pspxc)  #FIXME
         app("  maximum angular momentum: %s" % l2str(self.l_max))
         app("  angular momentum for local part: %s" % l2str(self.l_local))
         if self.isnc:
@@ -161,9 +164,9 @@ class Pseudo(object):
     def summary(self):
         """String summarizing the most important properties."""
 
-    @abc.abstractproperty
+    @property
     def filepath(self):
-        """Absolute path of the pseudopotential file."""
+        return os.path.abspath(self.path)
 
     @property
     def name(self):
@@ -181,7 +184,14 @@ class Pseudo(object):
     @property
     def element(self):
         """Pymatgen `Element`."""
-        return _PTABLE[self.Z]
+        try:
+            return _PTABLE[self.Z]
+        except (KeyError, IndexError):
+            return _PTABLE[int(self.Z)]
+
+    @property
+    def type(self):
+        return self.__class__.__name__
 
     @property
     def symbol(self):
@@ -196,6 +206,16 @@ class Pseudo(object):
     def l_local(self):
         """Angular momentum used for the local part."""
 
+    @property
+    def isnc(self):
+        """True if norm-conserving pseudopotential."""
+        return isinstance(self, NcPseudo)
+
+    @property
+    def ispaw(self):
+        """True if PAW pseudopotential."""
+        return isinstance(self, PawPseudo)
+
     #@abc.abstractproperty
     #def xc_type(self):
     #    """XC family e.g LDA, GGA, MGGA."""
@@ -209,16 +229,6 @@ class Pseudo(object):
     #    """XC identifier e.g LDA-PW91, GGA-PBE, GGA-revPBE."""
     #    return "-".join([self.xc_type, self.xc_flavor])
 
-    @property
-    def isnc(self):
-        """True if norm-conserving pseudopotential."""
-        return isinstance(self, NcPseudo)
-
-    @property
-    def ispaw(self):
-        """True if PAW pseudopotential."""
-        return isinstance(self, PawPseudo)
-
     #@abc.abstractproperty
     #def has_soc(self):
     #    """True if pseudo contains spin-orbit coupling."""
@@ -355,6 +365,14 @@ class NcPseudo(object):
         """True if the pseudo is generated with non-linear core correction."""
         return self.nlcc_radius > 0.0
 
+    @property
+    def rcore(self):
+        """Radius of the pseudization sphere in a.u."""
+        try:
+            return self._core
+        except AttributeError:
+            return None
+
 
 class PawPseudo(object):
     """
@@ -363,10 +381,28 @@ class PawPseudo(object):
     """
     __metaclass__ = abc.ABCMeta
 
+    #def nlcc_radius(self):
+    #    """
+    #    Radius at which the core charge vanish (i.e. cut-off in a.u.).
+    #    Returns 0.0 if nlcc is not used.
+    #    """
+    #    return 0.0
+    #                                                                           
+
+    #@property
+    #def has_nlcc(self):
+    #    """True if the pseudo is generated with non-linear core correction."""
+    #    return True
+
     @abc.abstractproperty
     def paw_radius(self):
         """Radius of the PAW sphere in a.u."""
 
+    @property
+    def rcore(self):
+        """Alias of paw_radius."""
+        return self.paw_radius
+
 
 class AbinitPseudo(Pseudo):
     """
@@ -400,10 +436,6 @@ class AbinitPseudo(Pseudo):
         """Summary line reported in the ABINIT header."""
         return self._summary.strip()
 
-    @property
-    def filepath(self):
-        return os.path.abspath(self.path)
-
     @property
     def Z(self):
         return self._zatom
@@ -429,10 +461,6 @@ class NcAbinitPseudo(NcPseudo, AbinitPseudo):
     def summary(self):
         return self._summary.strip()
 
-    @property
-    def filepath(self):
-        return os.path.abspath(self.path)
-
     @property
     def Z(self):
         return self._zatom
@@ -560,6 +588,7 @@ def _int_from_str(string):
     else:
         raise TypeError("Cannot convert string %s to int" % string)
 
+
 class NcAbinitHeader(AbinitHeader):
     """
     The abinit header found in the NC pseudopotential files.
@@ -636,6 +665,7 @@ class NcAbinitHeader(AbinitHeader):
 
         header["dojo_report"] = read_dojo_report(filename)
 
+        #print(header)
         return NcAbinitHeader(summary, **header)
 
     @staticmethod
@@ -903,14 +933,18 @@ class PseudoParser(object):
         paths = []
         for fname in os.listdir(dirname):
             root, ext = os.path.splitext(fname)
-            if ext in exclude_exts or fname in exclude_fnames or fname.startswith("."):
-                continue
-            paths.append(os.path.join(dirname, fname))
+            path = os.path.join(dirname, fname)
+            if (ext in exclude_exts or fname in exclude_fnames or 
+                fname.startswith(".") or not os.path.isfile(path)): continue
+            paths.append(path)
 
         pseudos = []
         for path in paths:
-            # parse the file and generate the pseudo.
-            pseudo = self.parse(path)
+            # Parse the file and generate the pseudo.
+            try:
+                pseudo = self.parse(path)
+            except:
+                pseudo = None
 
             if pseudo is not None:
                 pseudos.append(pseudo)
@@ -925,7 +959,7 @@ class PseudoParser(object):
         Read the pseudopotential descriptor from file filename.
 
         Returns:
-            Pseudopontential descriptor. None if filename is not a valid pseudopotential file.
+            Pseudopotential descriptor. None if filename is not a valid pseudopotential file.
 
         Raises:
             `PseudoParserError` if fileformat is not supported.
@@ -974,13 +1008,17 @@ class PseudoParser(object):
 
     def parse(self, filename):
         """
-        Read and parse a pseudopotential file.
+        Read and parse a pseudopotential file. Main entry point for client code.
 
         Returns: 
             pseudopotential object or None if filename is not a valid pseudopotential file.
         """
         path = os.path.abspath(filename)
 
+        # Only PAW supports XML at present.
+        if filename.endswith(".xml"):
+            return PawXmlSetup(path)
+
         ppdesc = self.read_ppdesc(path)
 
         if ppdesc is None: 
@@ -1006,12 +1044,11 @@ class PseudoParser(object):
 
         # Add the content of input file (if present).
         # The name of the input is name + ".ini"
-        input = None
-        input_path = root + ".ini"
-
-        if os.path.exists(input_path):
-            with open(input_path, 'r') as fh:
-                input = fh.read()
+        #input = None
+        #input_path = root + ".ini"
+        #if os.path.exists(input_path):
+        #    with open(input_path, 'r') as fh:
+        #        input = fh.read()
 
         if psp_type == "NC":
             pseudo = NcAbinitPseudo(path, header)
@@ -1023,6 +1060,428 @@ class PseudoParser(object):
         return pseudo
 
 
+#TODO use RadialFunction from pseudo_dojo.
+class RadialFunction(collections.namedtuple("RadialFunction", "mesh values")):
+    pass
+
+
+class PawXmlSetup(Pseudo, PawPseudo):
+    def __init__(self, filepath):
+        # FIXME
+        self.dojo_report = {}
+        self.path = os.path.abspath(filepath)
+
+        # Get the XML root (this trick is used to that the object is pickleable).
+        root = self.root
+
+        # Get the version of the XML format
+        self.paw_setup_version = root.get("version")
+
+        # Info on the atom.
+        atom_attrib = root.find("atom").attrib
+
+        #self._symbol = atom_attrib["symbol"]
+        self._zatom = int(float(atom_attrib["Z"]))
+        self.core, self.valence = map(float, [atom_attrib["core"], atom_attrib["valence"]])
+
+        #xc_info = root.find("atom").attrib
+        #self.xc_type, self.xc_name  = xc_info["type"], xc_info["name"]
+        #self.ae_energy = {k: float(v) for k,v in root.find("ae_energy").attrib.items()}
+
+        # Old XML files do not define this field!
+        # In this case we set the PAW radius to None.
+        #self._paw_radius = float(root.find("PAW_radius").attrib["rpaw"])
+
+        pawr_element = root.find("PAW_radius")
+        self._paw_radius = None
+        if pawr_element is not None:
+            self._paw_radius = float(pawr_element.attrib["rpaw"])
+
+        #<valence_states>
+        #  <state n="2" l="0" f="2"  rc="1.10" e="-0.6766" id="N-2s"/>
+        #  <state n="2" l="1" f="3"  rc="1.10" e="-0.2660" id="N-2p"/>
+        #  <state       l="0"        rc="1.10" e=" 0.3234" id="N-s1"/>
+        #  <state       l="1"        rc="1.10" e=" 0.7340" id="N-p1"/>
+        #  <state       l="2"        rc="1.10" e=" 0.0000" id="N-d1"/>
+        #</valence_states>
+        #
+        # The valence_states element contains several state elements.
+        # For this setup, the first two lines describe bound eigenstates
+        # with occupation numbers and principal quantum numbers.
+        # Notice, that the three additional unbound states should have no f and n attributes.
+        # In this way, we know that only the first two bound states (with f and n attributes)
+        # should be used for constructing an initial guess for the wave functions.
+
+        self.valence_states = {}
+        for node in root.find("valence_states"):
+            attrib = AttrDict(node.attrib)
+            assert attrib.id not in self.valence_states
+            self.valence_states[attrib.id] = attrib
+        #print(self.valence_states)
+
+        # Parse the radial grids
+        self.rad_grids = {}
+        for node in root.findall("radial_grid"):
+            grid_params = node.attrib
+            id = grid_params["id"]
+            assert id not in self.rad_grids
+
+            self.rad_grids[id] = self._eval_grid(grid_params)
+
+    def __getstate__(self):
+        """
+        Return state is pickled as the contents for the instance.
+                                                                                      
+        In this case we just remove the XML root element process since Element object cannot be pickled.
+        """
+        return {k:v for k,v in self.__dict__.items() if k not in ["_root",]}
+
+    @property
+    def root(self):
+        try:
+            return self._root
+        except AttributeError:
+            from xml.etree import cElementTree  as ET
+            tree = ET.parse(self.filepath)
+            self._root = tree.getroot()
+            return self._root
+
+    @property
+    def Z(self):
+        return self._zatom
+
+    @property
+    def Z_val(self):
+        """Number of valence electrons."""
+        return self.valence
+
+    # FIXME
+    @property
+    def l_max(self):
+        """Maximum angular momentum."""
+        return None
+                                                        
+    @property
+    def l_local(self):
+        """Angular momentum used for the local part."""
+        return None
+
+    @property
+    def summary(self):
+        """String summarizing the most important properties."""
+        return ""
+
+    @property
+    def paw_radius(self):
+        return self._paw_radius
+
+    @staticmethod
+    def _eval_grid(grid_params):
+        """
+        This function receives a dictionary with the parameters defining the
+        radial mesh and returns a `ndarray` with the mesh
+        """
+        eq = grid_params.get("eq").replace(" " ,"")
+        istart, iend = int(grid_params.get("istart")), int(grid_params.get("iend"))
+        indices = range(istart, iend+1)
+
+        if eq == 'r=a*exp(d*i)':
+            a, d = float(grid_params['a']), float(grid_params['d'])
+            mesh = [a * np.exp(d * i) for i in indices]
+
+        elif eq == 'r=a*i/(n-i)':
+            a, n = float(grid_params['a']), float(grid_params['n'])
+            mesh = [a * i / (n - i) for i in indices]
+
+        elif eq == 'r=a*(exp(d*i)-1)':
+            a, d = float(grid_params['a']), float(grid_params['d'])
+            mesh = [a * (np.exp(d * i) - 1.0) for i in indices]
+
+        elif eq == 'r=d*i':
+            d = float(grid_params['d'])
+            mesh = [d * i for i in indices]
+
+        elif eq == 'r=(i/n+a)^5/a-a^4':
+            a, n = float(grid_params['a']), float(grid_params['n'])
+            mesh = [(i / n + a)**5 / a - a**4 for i in indices]
+
+        else:
+            raise ValueError('Unknown grid type: %s' % eq)
+
+        return np.array(mesh)
+
+    def _parse_radfunc(self, func_name):
+        """Parse the first occurence of func_name in the XML file."""
+        node = self.root.find(func_name)
+        grid = node.attrib["grid"]
+        values = np.array([float(s) for s in node.text.split()])
+
+        return self.rad_grids[grid], values, node.attrib
+
+    def _parse_all_radfuncs(self, func_name):
+        """Parse all the nodes with tag func_name in the XML file."""
+        for node in self.root.findall(func_name):
+            grid = node.attrib["grid"]
+            values = np.array([float(s) for s in node.text.split()])
+
+            yield self.rad_grids[grid], values, node.attrib
+
+    @property
+    def ae_core_density(self):
+        """The all-electron radial density."""
+        try:
+            return self._ae_core_density
+
+        except AttributeError:
+            mesh, values, attrib = self._parse_radfunc("ae_core_density")
+            self._ae_core_density = RadialFunction(mesh, values)
+            return self._ae_core_density
+
+    @property
+    def pseudo_core_density(self):
+        """The pseudized radial density."""
+        try:
+            return self._pseudo_core_density
+
+        except AttributeError:
+            mesh, values, attrib = self._parse_radfunc("pseudo_core_density")
+            self._pseudo_core_density = RadialFunction(mesh, values)
+            return self._pseudo_core_density
+
+    @property
+    def ae_partial_waves(self):
+        """Dictionary with the AE partial waves indexed by state."""
+        try:
+            return self._ae_partial_waves
+
+        except AttributeError:
+            self._ae_partial_waves = {}
+            for (mesh, values, attrib) in self._parse_all_radfuncs("ae_partial_wave"):
+                state = attrib["state"]
+                val_state = self.valence_states[state]
+                self._ae_partial_waves[state] = RadialFunction(mesh, values)
+                #print("val_state", val_state)
+
+            return self._ae_partial_waves
+
+    @property
+    def pseudo_partial_waves(self):
+        """Dictionary with the pseudo partial waves indexed by state."""
+        try:
+            return self._pseudo_partial_waves
+
+        except AttributeError:
+            self._pseudo_partial_waves = {}
+            for (mesh, values, attrib) in self._parse_all_radfuncs("pseudo_partial_wave"):
+                state = attrib["state"]
+                val_state = self.valence_states[state]
+                self._pseudo_partial_waves[state] = RadialFunction(mesh, values)
+
+            return self._pseudo_partial_waves
+
+    @property
+    def projector_functions(self):
+        """Dictionary with the PAW projectors indexed by state."""
+        try:
+            return self._projector_functions
+
+        except AttributeError:
+            self._projector_functions = {}
+            for (mesh, values, attrib) in self._parse_all_radfuncs("projector_function"):
+                state = attrib["state"]
+                val_state = self.valence_states[state]
+                self._projector_functions[state] = RadialFunction(mesh, values)
+
+            return self._projector_functions
+
+    def plot_densities(self, **kwargs):
+        """
+        Plot the PAW densities.
+
+        ================  ==============================================================
+        kwargs            Meaning
+        ================  ==============================================================
+        title             Title of the plot (Default: "Densities").
+        show              True to show the figure (Default).
+        savefig           'abc.png' or 'abc.eps' to save the figure to a file.
+        ================  ==============================================================
+
+        Returns:
+            `matplotlib` figure
+        """
+        title = kwargs.pop("title", "Densities")
+        show = kwargs.pop("show", True)
+        savefig = kwargs.pop("savefig", None)
+
+        import matplotlib.pyplot as plt
+
+        fig = plt.figure()
+
+        ax = fig.add_subplot(1,1,1)
+        ax.grid(True)
+        ax.set_xlabel('r [Bohr]')
+        #ax.set_ylabel('density')
+
+        for i, den_name in enumerate(["ae_core_density", "pseudo_core_density"]):
+            rden = getattr(self, den_name)
+            label = "$n_c$" if i == 1 else "$\\tilde{n}_c$"
+            ax.plot(rden.mesh, rden.mesh * rden.values, label=label, lw=2)
+
+        plt.legend(loc="best")
+
+        if title is not None:
+            fig.suptitle(title)
+
+        if show:
+            plt.show()
+
+        if savefig:
+            fig.savefig(savefig)
+
+        return fig
+
+    def plot_waves(self, **kwargs):
+        """
+        Plot the AE and the pseudo partial waves.
+
+        ================  ==============================================================
+        kwargs            Meaning
+        ================  ==============================================================
+        title             Title of the plot (Default: "Partial Waves").
+        show              True to show the figure (Default).
+        savefig           'abc.png' or 'abc.eps' to save the figure to a file.
+        ================  ==============================================================
+
+        Returns:
+            `matplotlib` figure
+        """
+        title = kwargs.pop("title", "Partial Waves")
+        show = kwargs.pop("show", True)
+        savefig = kwargs.pop("savefig", None)
+
+        import matplotlib.pyplot as plt
+
+        fig = plt.figure()
+
+        ax = fig.add_subplot(1,1,1)
+        ax.grid(True)
+        ax.set_xlabel("r [Bohr]")
+        ax.set_ylabel("$r\phi,\\, r\\tilde\phi\, [Bohr]^{-\\frac{1}{2}}$")
+
+        ax.axvline(x=self.paw_radius, linewidth=2, color='k', linestyle="--")
+        #ax.annotate("$r_c$", xy=(self.paw_radius + 0.1, 0.1))
+
+        for state, rfunc in self.pseudo_partial_waves.items():
+            ax.plot(rfunc.mesh, rfunc.mesh * rfunc.values, lw=2, label="PS-WAVE: " + state)
+
+        for state, rfunc in self.ae_partial_waves.items():
+            ax.plot(rfunc.mesh, rfunc.mesh * rfunc.values, lw=2, label="AE-WAVE: " + state)
+
+        plt.legend(loc="best")
+
+        if title is not None:
+            fig.suptitle(title)
+
+        if show:
+            plt.show()
+
+        if savefig:
+            fig.savefig(savefig)
+
+        return fig
+
+    def plot_projectors(self, **kwargs):
+        """
+        Plot the PAW projectors.
+
+        ================  ==============================================================
+        kwargs            Meaning
+        ================  ==============================================================
+        title             Title of the plot (Default: "Projectors").
+        show              True to show the figure (Default).
+        savefig           'abc.png' or 'abc.eps' to save the figure to a file.
+        ================  ==============================================================
+
+        Returns:
+            `matplotlib` figure
+        """
+        title = kwargs.pop("title", "Projectors")
+        show = kwargs.pop("show", True)
+        savefig = kwargs.pop("savefig", None)
+
+        import matplotlib.pyplot as plt
+
+        fig = plt.figure()
+
+        ax = fig.add_subplot(1,1,1)
+        ax.grid(True)
+        ax.set_xlabel('r [Bohr]')
+        ax.set_ylabel("$r\\tilde p\, [Bohr]^{-\\frac{1}{2}}$")
+
+        ax.axvline(x=self.paw_radius, linewidth=2, color='k', linestyle="--")
+        #ax.annotate("$r_c$", xy=(self.paw_radius + 0.1, 0.1))
+
+        for state, rfunc in self.projector_functions.items():
+            ax.plot(rfunc.mesh, rfunc.mesh * rfunc.values, label="TPROJ: " + state)
+
+        plt.legend(loc="best")
+
+        if title is not None:
+            fig.suptitle(title)
+
+        if show:
+            plt.show()
+
+        if savefig:
+            fig.savefig(savefig)
+
+        return fig
+
+    #def plot_potentials(self, **kwargs):
+    #    """
+    #        ================  ==============================================================
+    #        kwargs            Meaning
+    #        ================  ==============================================================
+    #        title             Title of the plot (Default: None).
+    #        show              True to show the figure (Default).
+    #        savefig           'abc.png' or 'abc.eps' to save the figure to a file.
+    #        ================  ==============================================================
+
+    #    Returns:
+    #        `matplotlib` figure
+    #    """
+    #    title = kwargs.pop("title", "Potentials")
+    #    show = kwargs.pop("show", True)
+    #    savefig = kwargs.pop("savefig", None)
+
+    #    import matplotlib.pyplot as plt
+
+    #    fig = plt.figure()
+
+    #    ax = fig.add_subplot(1,1,1)
+    #    ax.grid(True)
+    #    ax.set_xlabel('r [Bohr]')
+    #    ax.set_ylabel('density')
+    #    ax.axvline(x=self.paw_radius, linewidth=2, color='k', linestyle="--")
+    #    ax.annotate("$r_c$", xy=(self.paw_radius + 0.1, 0.1))
+
+    #    for state, rfunc in self.potentials.items():
+    #        ax.plot(rfunc.mesh, rfunc.values, label="TPROJ: " + state)
+
+    #    plt.legend(loc="best")
+
+    #    if title is not None:
+    #        fig.suptitle(title)
+
+    #    if show:
+    #        plt.show()
+
+    #    if savefig:
+    #        fig.savefig(savefig)
+
+    #    return fig
+
+
 class PseudoTable(collections.Sequence):
     """
     Define the pseudopotentials from the element table.
@@ -1152,6 +1611,7 @@ class PseudoTable(collections.Sequence):
         try:
             return getattr(self, str(symbol))
         except AttributeError:
+            #raise
             return []
 
     def pseudo_from_name(self, name):
@@ -1205,25 +1665,25 @@ class PseudoTable(collections.Sequence):
                     print("format",format,"args",values)
                     raise
 
-    def print_table(self, stream=sys.stdout, filter_function=None):
-        """
-        A pretty ASCII printer for the periodic table, based on some filter_function.
+    #def print_table(self, stream=sys.stdout, filter_function=None):
+    #    """
+    #    A pretty ASCII printer for the periodic table, based on some filter_function.
 
-        Args:
-            filter_function:
-                A filtering function that take a Pseudo as input and returns a boolean.
-                For example, setting filter_function = lambda el: el.Z_val > 2 will print
-                a periodic table containing only pseudos with Z_val > 2.
-        """
-        for row in range(1, 10):
-            rowstr = []
-            for group in range(1, 19):
-                el = Element.from_row_and_group(row, group)
-                if el and ((not filter_function) or filter_function(el)):
-                    rowstr.append("{:3s}".format(el.symbol))
-                else:
-                    rowstr.append("   ")
-            print(" ".join(rowstr))
+    #    Args:
+    #        filter_function:
+    #            A filtering function that take a Pseudo as input and returns a boolean.
+    #            For example, setting filter_function = lambda el: el.Z_val > 2 will print
+    #            a periodic table containing only pseudos with Z_val > 2.
+    #    """
+    #    for row in range(1, 10):
+    #        rowstr = []
+    #        for group in range(1, 19):
+    #            el = Element.from_row_and_group(row, group)
+    #            if el and ((not filter_function) or filter_function(el)):
+    #                rowstr.append("{:3s}".format(el.symbol))
+    #            else:
+    #                rowstr.append("   ")
+    #        print(" ".join(rowstr))
 
     def sorted(self, attrname, reverse=False):
         """Sort the table according to the value of attribute attrname."""
@@ -1252,4 +1712,3 @@ class PseudoTable(collections.Sequence):
     def with_dojo_report(self):
         """Select pseudos containing the DOJO_REPORT section."""
         return self.select(condition=lambda p : p.has_dojo_report)
-
