@@ -41,6 +41,7 @@ __maintainer__ = "Matteo Giantomassi"
 
 __all__ = [
     "TaskManager",
+    "ParalHintsParser",
     "ScfTask",
     "NscfTask",
     "RelaxTask",
@@ -52,6 +53,11 @@ __all__ = [
     "AnaddbTask",
 ]
 
+# Tools and helper functions.
+def straceback():
+    """Returns a string with the traceback."""
+    import traceback
+    return traceback.format_exc()
 
 class TaskResults(dict, MSONable):
     """
@@ -141,7 +147,7 @@ class ParalConf(AttrDict):
             -   tot_ncpus: 2         # Total number of CPUs
                 mpi_ncpus: 2         # Number of MPI processes.
                 omp_ncpus: 1         # Number of OMP threads (1 if not present)
-                mem_per_cpus: 10     # Estimated memory requirement per MPI processor in Gigabytes (None if not specified)
+                mem_per_cpu: 10     # Estimated memory requirement per MPI processor in Megabytes.
                 efficiency: 0.4      # 1.0 corresponds to an "expected" optimal efficiency (strong scaling).
                 vars: {              # Dictionary with the variables that should be added to the input.
                       varname1: varvalue1
@@ -176,6 +182,12 @@ class ParalConf(AttrDict):
         """Estimated speedup reported by ABINIT."""
         return self.efficiency * self.tot_ncpus
 
+    @property
+    def tot_mem(self):
+        """Estimated total memory in Mbs (computed from mem_per_cpu)"""
+        return self.mem_per_cpu * self.tot_ncpus
+
+
 
 class ParalHintsParser(object):
 
@@ -189,9 +201,12 @@ class ParalHintsParser(object):
         with abiinspect.YamlTokenizer(filename) as r:
             doc = r.next_doc_with_tag("!Autoparal")
 
+            #print(doc.text)
+            #with open(os.path.join(os.path.dirname(filename), "autoparal.yml"), "w") as fh:
+            #    fh.write(doc.text)
+
             try:
                 d = yaml.load(doc.text_notag)
-                #print(d)
                 return ParalHints(info=d["info"], confs=d["configurations"])
 
             except:
@@ -224,9 +239,18 @@ class ParalHints(collections.Iterable):
         return copy.copy(self)
 
     def select_with_condition(self, condition):
+        """
+        Remove all the configurations that do not satisfy the given condition.
+
+            Args:
+                `Condition` object with operators expressed with a Mongodb-like syntax
+        """
         new_confs = []
+
         for conf in self:
-            if condition.apply(obj=conf):
+            add_it = condition.apply(obj=conf)
+            #print("add_it", add_it, "conf", conf)
+            if add_it:
                 new_confs.append(conf)
 
         self._confs = new_confs
@@ -245,42 +269,62 @@ class ParalHints(collections.Iterable):
         """
         self._confs.sort(key=lambda c: c.speedup, reverse=reverse)
 
-    #def sort_by_mem_cpu(self, reverse=False):
-    #    self._confs.sort(key=lambda c: c.mem_per_cpus, reverse=reverse)
+    def sort_by_mem_cpu(self, reverse=False):
+        """
+        Sort the configurations in place so that conf with lowest memory per CPU
+        appears in the first positions.
+        """
+        # Avoid sorting if mem_per_cpu is not available.
+        has_mem_info = any(c.mem_per_cpu > 0.0 for c in self)
+
+        if has_mem_info:
+            self._confs.sort(key=lambda c: c.mem_per_cpu, reverse=reverse)
 
     def select_optimal_conf(self, policy):
-        """Find the optimal configuration according on policy."""
+        """
+        Find the optimal configuration according to the `TaskPolicy` policy.
+        """
         # Make a copy since we are gonna change the object in place.
-        hints = self.copy()
+        #hints = self.copy()
+
+        hints = ParalHints(self.info, confs=[c for c in self if c.tot_ncpus <= policy.max_ncpus])
+        #print(hints)
 
         # First select the configurations satisfying the 
         # condition specified by the user (if any)
         if policy.condition:
+            print("condition",policy.condition)
             hints.select_with_condition(policy.condition)
-            #if not hints: hints = self.copy()
-            #print("after condition", hints)
+            print("after condition", hints)
+
+            # If no configuration fullfills the requirements, 
+            # we return the one with the highest speedup.
+            if not hints:
+                #print("no configuration")
+                hints = self.copy()
+                hints.sort_by_speedup()
+                return hints[-1].copy()
 
         hints.sort_by_speedup()
 
-        # If no configuration fullfills the requirements, 
-        # we return the one with the highest speedup.
-        if not hints:
-            hints = self.copy()
-            hints.sort_by_speedup()
-            return hints[-1].copy()
-
         # Find the optimal configuration according to policy.mode.
         #mode = policy.mode
         #if mode in ["default", "aggressive"]:
-        #    hints.sort_by_spedup(reverse=True)
+        #    hints.sort_by_spedup()
+
         #elif mode == "conservative":
-        #    hints.sort_by_efficiency(reverse=True)
+        #    hints.sort_by_efficiency()
+        #    # Remove tot_ncpus == 1
+        #    hints.pop(tot_ncpus==1)
+        #    if not hints:
+
         #else:
         #    raise ValueError("Wrong value for mode: %s" % str(mode))
 
         # Return a copy of the configuration.
         optimal = hints[-1].copy()
-        logger.debug("Will relaunch the job with optimized parameters:\n %s" % str(optimal))
+        logger.debug("Will relaunch the job with optimized parameters:\n %s" % optimal)
+
         return optimal
 
 
@@ -293,11 +337,16 @@ class TaskPolicy(object):
     and the condition used to select the optimal configuration for the parallel run 
     """
 
-    def __init__(self, autoparal=0, mode="default", max_ncpus=None, use_fw=False, condition=None): 
+    def __init__(self, autoparal=0, automemory=0, mode="default", max_ncpus=None, use_fw=False, condition=None): 
         """
         Args:
             autoparal: 
                 Value of ABINIT autoparal input variable. None to disable the autoparal feature.
+            automemory:
+                int defining the memory policy. 
+                If > 0 the memory requirements will be computed at run-time from the autoparal section
+                produced by ABINIT. In this case, the job script will report the autoparal memory
+                instead of the one specified by the user.
             mode:
                 Select the algorith to select the optimal configuration for the parallel execution.
                 Possible values: ["default", "aggressive", "conservative"]
@@ -309,6 +358,7 @@ class TaskPolicy(object):
                 condition used to filter the autoparal configuration (Mongodb syntax)
         """
         self.autoparal = autoparal
+        self.automemory = automemory
         self.mode = mode 
         self.max_ncpus = max_ncpus
         self.use_fw = use_fw 
@@ -369,6 +419,7 @@ class TaskManager(object):
 
     @classmethod
     def from_dict(cls, d):
+        """Create an instance from a dictionary."""
         return cls(**d)
 
     @classmethod
@@ -381,7 +432,7 @@ class TaskManager(object):
     def from_user_config(cls):
         """
         Initialize the `TaskManager` from the YAML file 'taskmanager.yaml'.
-        Search first in the workind directory and then in the configuration
+        Search first in the working directory and then in the configuration
         directory of abipy.
 
         Raises:
@@ -451,6 +502,14 @@ class TaskManager(object):
 
         return new
 
+    def new_with_policy(self, policy):
+        """
+        Returns a new `TaskManager` with same parameters as self except for policy.
+        """
+        new = self.deepcopy()
+        new.policy = policy
+        return new
+
     def copy(self):
         """Shallow copy of self."""
         return copy.copy(self)
@@ -467,92 +526,9 @@ class TaskManager(object):
         """Set the number of OpenMp threads to use."""
         self.qadapter.set_omp_ncpus(omp_ncpus)
 
-    # TODO
-    #def set_mem_per_cpu(self, mem_per_cpu):
-    #    """Set the memory (in gigabytes) per CPU."""
-    #    self.qadapter.set_mem_per_cpu(mem_per_cpu)
-
-    def autoparal(self, task):
-        """
-        Find an optimal set of parameters for the execution of the task 
-        using the options specified in `TaskPolicy`.
-        This method can change the ABINIT input variables and/or the 
-        parameters passed to the `TaskManager` e.g. the number of CPUs for MPI and OpenMp.
-
-        Returns:
-           confs, optimal 
-           where confs is a `ParalHints` object with the configuration reported by 
-           autoparal and optimal is the optimal configuration selected.
-           Returns (None, None) if some problem occurred.
-        """
-        policy = self.policy
-
-        if policy.autoparal == 0 or policy.max_ncpus in [None, 1]: 
-            logger.info("Nothing to do in autoparal, returning (None, None)")
-            return None, None
-
-        assert policy.autoparal == 1
-        # 1) Run ABINIT in sequential to get the possible configurations with max_ncpus
-
-        # Set the variables for automatic parallelization
-        autoparal_vars = dict(
-            autoparal=policy.autoparal,
-            max_ncpus=policy.max_ncpus,
-        )
-
-        task.strategy.add_extra_abivars(autoparal_vars)
-        task.build()
-
-        # Build a simple manager to run the job in a shell subprocess on the frontend
-        # we don't want to make a request to the queue manager for this simple job!
-        seq_manager = self.to_shell_manager(mpi_ncpus=1)
-
-        # Return code is always != 0 
-        process = seq_manager.launch(task)
-        process.wait()  
-
-        # Remove the variables added for the automatic parallelization
-        task.strategy.remove_extra_abivars(autoparal_vars.keys())
-
-        # 2) Parse the autoparal configurations
-        parser = ParalHintsParser()
-
-        try:
-            confs = parser.parse(task.log_file.path)
-
-        except parser.Error:
-            return None, None
-
-        # 3) Select the optimal configuration according to policy
-        optimal = confs.select_optimal_conf(policy)
-        print("optimal Autoparal conf:\n %s" % optimal)
-
-        # 4) Change the input file and/or the submission script
-        task.strategy.add_extra_abivars(optimal.vars)
-                                                                  
-        # Change the number of MPI nodes.
-        self.set_mpi_ncpus(optimal.mpi_ncpus)
-
-        # Change the number of OpenMP threads.
-        #if optimal.omp_ncpus > 1:
-        #    self.set_omp_ncpus(optimal.omp_ncpus)
-        #else:
-        #    self.qadapter.disable_omp()
-
-        # Change the memory per node.
-        #if optimal.mem_per_cpu is not None
-        #    self.qadapter.set_mem_per_cpu(optimal.mem_per_cpu)
-
-        # Reset the status, remove garbage files ...
-        task.set_status(task.S_READY)
-
-        # Remove the output file since Abinit likes to create new files 
-        # with extension .outA, .outB if the file already exists.
-        os.remove(task.output_file.path)
-        os.remove(task.log_file.path)
-        os.remove(task.stderr_file.path)
-
-        return confs, optimal
+    def set_mem_per_cpu(self, mem_mb):
+        """Set the memory (in Megabytes) per CPU."""
+        self.qadapter.set_mem_per_cpu(mem_mb)
 
     def write_jobfile(self, task):
         """
@@ -807,6 +783,7 @@ STATUS2STR = collections.OrderedDict([
 ])
 
 class Status(int):
+    """This object is an integer representing the status of the `Node`."""
     def __repr__(self):
         return "<%s: %s, at %s>" % (self.__class__.__name__, str(self), id(self))
 
@@ -820,7 +797,7 @@ class Node(object):
     implemented by the nodes of the calculation.
 
     Nodes are hashable and can be tested for equality
-    (both operation use the node identifier).
+    (hash uses the node identifier, while eq uses workdir).
     """
     __metaclass__ = abc.ABCMeta
 
@@ -852,7 +829,9 @@ class Node(object):
         if not isinstance(other, Node):
             return False
 
-        return self.node_id == other.node_id and self.__class__ == other.__class__
+        return (self.__class__ == other.__class__ and 
+                self.workdir == other.workdir)
+               #self.node_id == other.node_id and 
                                                        
     def __ne__(self, other):
         return not self.__eq__(other)
@@ -879,7 +858,7 @@ class Node(object):
     @property
     def name(self):
         """
-        The name of node 
+        The name of the node 
         (only used for facilitating its identification in the user interface).
         """
         try:
@@ -909,6 +888,11 @@ class Node(object):
     def finalize(self, boolean):
         self._finalized = boolean
         self.history.append("Finalized on %s" % time.asctime())
+
+    @property
+    def str_history(self):
+        """String representation of history."""
+        return "\n".join(self.history)
                                                          
     #@abc.abstractproperty
     #def workdir(self):
@@ -1024,6 +1008,7 @@ class FakeDirectory(Directory):
             return self.filepath
         return ""
 
+
 class FileNode(Node):
     """
     A node of the calculation consisting of an already existing file
@@ -1116,9 +1101,8 @@ class Task(Node):
         # Set the initial status.
         self.set_status(self.S_INIT)
 
-        # Number of restarts effectuated and max number (-1 --> no limit).
+        # Number of restarts effectuated.
         self.num_restarts = 0
-        self.max_num_restarts = -1
 
     def __getstate__(self):
         """
@@ -1130,9 +1114,11 @@ class Task(Node):
         """
         return {k:v for k,v in self.__dict__.items() if k not in ["_process",]}
 
-    def set_workdir(self, workdir):
-        """Set the working directory. Cannot be set more than once."""
-        assert not hasattr(self, "workdir")
+    def set_workdir(self, workdir, chroot=False):
+        """Set the working directory. Cannot be set more than once unless chroot is True"""
+        if not chroot and hasattr(self, "workdir") and self.workdir != workdir:
+                raise ValueError("self.workdir != workdir: %s, %s" % (self.workdir,  workdir))
+
         self.workdir = os.path.abspath(workdir)
 
         # Files required for the execution.
@@ -1142,6 +1128,7 @@ class Task(Node):
         self.job_file = File(os.path.join(self.workdir, "job.sh"))
         self.log_file = File(os.path.join(self.workdir, "run.log"))
         self.stderr_file = File(os.path.join(self.workdir, "run.err"))
+        self.start_lockfile = File(os.path.join(self.workdir, "__startlock__"))
 
         # Directories with input|output|temporary data.
         self.indir = Directory(os.path.join(self.workdir, "indata"))
@@ -1221,7 +1208,7 @@ class Task(Node):
     @property
     def can_run(self):
         """The task can run if its status is < S_SUB and all the other depencies (if any) are done!"""
-        all_ok = all(stat == self.S_OK for stat in self.deps_status)
+        all_ok = all([stat == self.S_OK for stat in self.deps_status])
         return self.status < self.S_SUB and all_ok
 
     def not_converged(self):
@@ -1230,6 +1217,24 @@ class Task(Node):
         report = self.get_event_report()
         return report.filter_types(self.CRITICAL_EVENTS)
 
+    def cancel(self):
+        """
+        Cancel the job. Returns 1 if job was cancelled.
+        """
+        queue_id = self.queue_id
+        if queue_id is None: return 0 
+        if self.status >= self.S_DONE: return 0 
+
+        exit_status = self.manager.qadapter.cancel(queue_id)
+
+        if exit_status != 0:
+            return 0
+
+        # Remove output files and reset the status.
+        self.reset()
+
+        return 1
+
     def _on_done(self):
         self.fix_ofiles()
 
@@ -1276,49 +1281,31 @@ class Task(Node):
         """
         self.set_status(self.S_READY, info_msg="Restarted on %s" % time.asctime())
 
-        # Increase the counter and relaunch the task.
+        # Increase the counter.
         self.num_restarts += 1
         self.history.append("Restarted on %s, num_restarts %d" % (time.asctime(), self.num_restarts))
-        self.start()
 
-        return 0
+        # Remove the lock file
+        self.start_lockfile.remove()
+ 
+        # Relaunch the task.
+        fired = self.start()
+
+        if not fired:
+            self.history.append("[%s], restart failed" % time.asctime())
+
+        return fired
 
     def restart(self):
         """
-        Restart the calculation. This method is called if the calculation is not converged 
-        and we can restart the task. See restart_if_needed.
-        """
-        logger.debug("Calling the **empty** restart method of the base class")
-
-    def restart_if_needed(self):
-        """
-        Callback that is executed once the job is done. 
-        The implementation of the two methods: 
-
-           - not_converged
-           - restart 
-           
-        is delegated to the subclasses.
+        Restart the calculation.  Subclasses should provide a concrete version that 
+        performs all the actions needed for preparing the restart and then calls self._restart
+        to restart the task. The default implementation is empty.
 
         Returns:
-            0 if succes, 1 if restart was not possible.
+            1 if job was restarted, 0 otherwise.
         """
-        if self.not_converged():
-            if self.num_restarts == self.max_num_restarts:
-                info_msg = "Reached maximum number of restarts. Cannot restart anymore Returning"
-                logger.info(info_msg)
-                self.history.append(info_msg)
-                return 1
-
-            try:
-                self.restart()
-    
-            except TaskRestartError as exc:
-                info_msg = "Calculation not converged but restart was not possible!\nException: %s" % exc
-                logger.debug(info_msg)
-                self.set_status(self.S_ERROR, info_msg=info_msg)
-                return 1
-     
+        logger.debug("Calling the **empty** restart method of the base class")
         return 0
 
     def poll(self):
@@ -1381,6 +1368,15 @@ class Task(Node):
             return 1
 
         self.set_status(self.S_INIT, info_msg="Reset on %s" % time.asctime())
+        self.set_queue_id(None)
+
+        # Remove output files otherwise the EventParser will think the job is still running
+        self.output_file.remove()
+        self.log_file.remove()
+        self.stderr_file.remove()
+        self.start_lockfile.remove()
+        self.qerr_file.remove()
+        self.qout_file.remove()
 
         # TODO send a signal to the flow 
         #self.workflow.check_status()
@@ -1813,34 +1809,53 @@ class Task(Node):
             - build dirs and files
             - call the _setup method
             - execute the job file by executing/submitting the job script.
+
+        Returns:
+            1 if task was started, 0 otherwise.
+            
         """
-        if self._status >= self.S_SUB:
+        if self.status >= self.S_SUB:
             raise self.Error("Task status: %s" % str(self.status))
 
+        if self.start_lockfile.exists:
+            logger.critical("Found lock file: %s" % self.start_lockfile.relpath)
+            return 0
+
+        self.start_lockfile.write("Started on %s" % time.asctime())
+
         self.build()
         self._setup()
 
         # Add the variables needed to connect the node.
         for d in self.deps:
             vars = d.connecting_vars()
-            logger.debug("Adding connecting vars %s " % str(vars))
+            logger.debug("Adding connecting vars %s " % vars)
             self.strategy.add_extra_abivars(vars)
 
         # Automatic parallelization
-        self.manager.autoparal(self)
+        if hasattr(self, "autoparal_fake_run"):
+            try:
+                self.autoparal_fake_run()
+            except:
+                # Log the exception and continue with the parameters specified by the user.
+                logger.critical("autoparal_fake_run raised:\n%s" % straceback())
+                self.set_status(self.S_ERROR)
+                return 0
 
         # Start the calculation in a subprocess and return.
         self._process = self.manager.launch(self)
 
-    def start_and_wait(self, *args, **kwargs):
-        """
-        Helper method to start the task and wait.
+        return 1
 
-        Mainly used when we are submitting the task via the shell
-        without passing through a queue manager.
-        """
-        self.start(*args, **kwargs)
-        return self.wait()
+    #def start_and_wait(self, *args, **kwargs):
+    #    """
+    #    Helper method to start the task and wait.
+
+    #    Mainly used when we are submitting the task via the shell
+    #    without passing through a queue manager.
+    #    """
+    #    self.start(*args, **kwargs)
+    #    return self.wait()
 
 
 class AbinitTask(Task):
@@ -1935,6 +1950,102 @@ class AbinitTask(Task):
 
         return "\n".join(lines)
 
+    def autoparal_fake_run(self):
+        """
+        Find an optimal set of parameters for the execution of the task 
+        using the options specified in `TaskPolicy`.
+        This method can change the ABINIT input variables and/or the 
+        parameters passed to the `TaskManager` e.g. the number of CPUs for MPI and OpenMp.
+
+        Returns:
+           confs, optimal 
+           where confs is a `ParalHints` object with the configuration reported by 
+           autoparal and optimal is the optimal configuration selected.
+           Returns (None, None) if some problem occurred.
+        """
+        logger.info("in autoparal_fake_run")
+        manager = self.manager
+        policy = manager.policy
+
+        if policy.autoparal == 0 or policy.max_ncpus in [None, 1]: 
+            msg = "Nothing to do in autoparal, returning (None, None)"
+            logger.info(msg)
+            print(msg)
+            return None, None
+
+        if policy.autoparal != 1:
+            raise NotImplementedError("autoparal != 1")
+
+        # 1) Run ABINIT in sequential to get the possible configurations with max_ncpus
+
+        # Set the variables for automatic parallelization
+        autoparal_vars = dict(
+            autoparal=policy.autoparal,
+            max_ncpus=policy.max_ncpus,
+        )
+
+        self.strategy.add_extra_abivars(autoparal_vars)
+
+        # Build a simple manager to run the job in a shell subprocess on the frontend
+        # we don't want to make a request to the queue manager for this simple job!
+        seq_manager = manager.to_shell_manager(mpi_ncpus=1)
+
+        # Return code is always != 0 
+        process = seq_manager.launch(self)
+        #print("launched")
+        process.wait()  
+
+        # Remove the variables added for the automatic parallelization
+        self.strategy.remove_extra_abivars(autoparal_vars.keys())
+
+        # 2) Parse the autoparal configurations
+        #print("parsing")
+        parser = ParalHintsParser()
+
+        try:
+            confs = parser.parse(self.log_file.path)
+            #print("confs", confs)
+            #self.all_autoparal_confs = confs 
+
+        except parser.Error:
+            logger.critical("Error while parsing Autoparal section:\n%s" % straceback())
+            return None, None
+
+        # 3) Select the optimal configuration according to policy
+        optimal = confs.select_optimal_conf(policy)
+        print("optimal Autoparal conf:\n %s" % optimal)
+
+        # 4) Change the input file and/or the submission script
+        self.strategy.add_extra_abivars(optimal.vars)
+                                                                  
+        # Change the number of MPI nodes.
+        manager.set_mpi_ncpus(optimal.mpi_ncpus)
+
+        # Change the number of OpenMP threads.
+        #if optimal.omp_ncpus > 1:
+        #    manager.set_omp_ncpus(optimal.omp_ncpus)
+        #else:
+        #    manager.disable_omp()
+
+        # Change the memory per node if automemory evaluates to True.
+        mem_per_cpu = optimal.mem_per_cpu
+
+        if policy.automemory and mem_per_cpu:
+            # mem_per_cpu = max(mem_per_cpu, policy.automemory)
+            manager.set_mem_per_cpu(mem_per_cpu)
+
+        # Reset the status, remove garbage files ...
+        self.set_status(self.S_INIT)
+
+        # Remove the output file since Abinit likes to create new files 
+        # with extension .outA, .outB if the file already exists.
+        os.remove(self.output_file.path)
+        os.remove(self.log_file.path)
+        os.remove(self.stderr_file.path)
+
+        return confs, optimal
+
+
 # TODO
 # Enable restarting capabilites:
 # Before doing so I need:
@@ -1971,7 +2082,7 @@ class ScfTask(AbinitTask):
         self.strategy.add_extra_abivars(irdvars)
 
         # Now we can resubmit the job.
-        self._restart()
+        return self._restart()
 
     def inspect(self, **kwargs):
         """
@@ -2010,7 +2121,7 @@ class NscfTask(AbinitTask):
         self.strategy.add_extra_abivars(irdvars)
 
         # Now we can resubmit the job.
-        self._restart()
+        return self._restart()
 
 
 class RelaxTask(AbinitTask):
@@ -2076,7 +2187,7 @@ class RelaxTask(AbinitTask):
         self.strategy.add_extra_abivars(irdvars)
 
         # Now we can resubmit the job.
-        self._restart()
+        return self._restart()
 
     def inspect(self, **kwargs):
         """
@@ -2126,7 +2237,7 @@ class PhononTask(AbinitTask):
         self.strategy.add_extra_abivars(irdvars)
 
         # Now we can resubmit the job.
-        self._restart()
+        return self._restart()
 
     def inspect(self, **kwargs):
         """
@@ -2165,7 +2276,7 @@ class G_Task(AbinitTask):
         self.strategy.add_extra_abivars(irdvars)
 
         # Now we can resubmit the job.
-        self._restart()
+        return self._restart()
 
 
 class BseTask(AbinitTask):
@@ -2214,8 +2325,7 @@ class HaydockBseTask(BseTask):
 
         if not count:
             # outdir does not contain the BSR|BSC file.
-            # This means that num_restart > 1 and the files should
-            # be in task.indir
+            # This means that num_restart > 1 and the files should be in task.indir
             count = 0
             for ext in ["BSR", "BSC"]:
                 ifile = self.indir.has_abiext(ext)
@@ -2241,7 +2351,7 @@ class HaydockBseTask(BseTask):
         self.strategy.add_extra_abivars(irdvars)
 
         # Now we can resubmit the job.
-        self._restart()
+        return self._restart()
 
 
 class OpticTask(Task):
@@ -2458,3 +2568,5 @@ class AnaddbTask(Task):
         Anaddb allows the user to specify the paths of the input file.
         hence we don't need to create symbolic links.
         """
+
+
