@@ -6,7 +6,7 @@ import collections
 import shutil
 import operator
 
-from pymatgen.util.string_utils import list_strings, StringColorizer
+from pymatgen.util.string_utils import list_strings, StringColorizer, WildCard
 
 import logging
 logger = logging.getLogger(__name__)
@@ -43,6 +43,11 @@ class File(object):
         """File basename."""
         return os.path.basename(self.path)
 
+    @property
+    def relpath(self):
+        """Relative path."""
+        return os.path.relpath(self.path)
+
     @property
     def dirname(self):
         """Absolute path of the directory where the file is located."""
@@ -85,6 +90,13 @@ class File(object):
         if not os.path.exists(self.dirname):
             os.makedirs(self.dirname)
 
+    def remove(self):
+        """Remove the file."""
+        try:
+            os.remove(self.path)
+        except:
+            pass
+
 
 class Directory(object):
     """
@@ -112,6 +124,11 @@ class Directory(object):
         """Absolute path of the directory."""
         return self._path
 
+    @property
+    def relpath(self):
+        """Relative path."""
+        return os.path.relpath(self.path)
+
     @property
     def basename(self):
         """Directory basename."""
@@ -126,7 +143,7 @@ class Directory(object):
 
     @property
     def exists(self):
-        "True if file exists."
+        """True if file exists."""
         return os.path.exists(self.path)
 
     def makedirs(self):
@@ -146,10 +163,31 @@ class Directory(object):
         """Return the absolute path of filename in the directory."""
         return os.path.join(self.path, filename)
 
-    def list_filepaths(self):
-        """Return the list of absolute filepaths in the top-level directory."""
+    def list_filepaths(self, wildcard=None):
+        """
+        Return the list of absolute filepaths in the directory.
+
+        Args:
+            wildcard:
+                String of tokens separated by "|".
+                Each token represents a pattern.
+                If wildcard is not None, we return only those files that
+                match the given shell pattern (uses fnmatch).
+
+                Example:
+                  wildcard="*.nc|*.pdf" selects only those files that end with .nc or .pdf
+        """
+        # Selecte the files in the directory.
         fnames = [f for f in os.listdir(self.path)]
-        return [os.path.join(self.path, f) for f in fnames]
+        filepaths = filter(os.path.isfile, [os.path.join(self.path, f) for f in fnames])
+
+        # Filter using the shell patterns.
+        if wildcard is not None:
+            filepaths =  WildCard(wildcard).filter(filepaths)
+            #print(filepaths)
+
+        return filepaths
+
 
     def has_abiext(self, ext):
         """
@@ -368,7 +406,7 @@ _BIN_OPS = {
     }
 
 
-ops = _UNARY_OPS.keys() + _BIN_OPS.keys()
+_ALL_OPS = list(_UNARY_OPS.keys()) + list(_BIN_OPS.keys())
 
 
 def map2rpn(map, obj):
@@ -388,7 +426,7 @@ def map2rpn(map, obj):
 
     for k, v in map.items():
 
-        if k in ops:
+        if k in _ALL_OPS:
             if isinstance(v, collections.Mapping):
                 # e.g "$not": {"$gt": "one"}
                 # print("in op_vmap",k, v)
@@ -445,7 +483,7 @@ def evaluate_rpn(rpn):
                                                                               
     for item in rpn:
                                                                               
-        if item in ops:
+        if item in _ALL_OPS:
             # Apply the operator and push to the task.
             v2 = vals_stack.pop()
                                                                               
@@ -456,7 +494,7 @@ def evaluate_rpn(rpn):
                 v1 = vals_stack.pop()
                 res = _BIN_OPS[item](v1, v2)
             else:
-                raise ValuError("%s not in unary_ops or bin_ops" % str(item))
+                raise ValueError("%s not in unary_ops or bin_ops" % str(item))
                                                                               
             vals_stack.append(res)
                                                                               
