@@ -50,6 +50,7 @@ __all__ = [
     "RelaxWorkflow",
     "DeltaFactorWorkflow",
     "G0W0_Workflow",
+    "SigmaConvWorkflow",
     "BSEMDF_Workflow",
     "PhononWorkflow",
 ]
@@ -167,6 +168,13 @@ class BaseWorkflow(Node):
         logger.warning("Possible deadlock in fetch_task_to_run!")
         return None
 
+    def fetch_alltasks_to_run(self):
+        """
+        Returns a list with all the tasks that can be submitted.
+        Empty list if not task has been found.
+        """
+        return [task for task in self if task.can_run]
+
     @abc.abstractmethod
     def setup(self, *args, **kwargs):
         """Method called before submitting the calculations."""
@@ -277,10 +285,9 @@ class Workflow(BaseWorkflow):
             if self._flow != flow:
                 raise ValueError("self._flow != flow")
 
-    def set_workdir(self, workdir):
-        """Set the working directory. Cannot be set more than once."""
-
-        if hasattr(self, "workdir") and self.workdir != workdir:
+    def set_workdir(self, workdir, chroot=False):
+        """Set the working directory. Cannot be set more than once unless chroot is True"""
+        if not chroot and hasattr(self, "workdir") and self.workdir != workdir:
             raise ValueError("self.workdir != workdir: %s, %s" % (self.workdir,  workdir))
 
         self.workdir = os.path.abspath(workdir)
@@ -293,6 +300,13 @@ class Workflow(BaseWorkflow):
         self.outdir = Directory(os.path.join(self.workdir, "outdata"))
         self.tmpdir = Directory(os.path.join(self.workdir, "tmpdata"))
 
+    def chroot(self, new_workdir):
+        self.set_workdir(new_workdir, chroot=True)
+
+        for i, task in enumerate(self):
+            new_tdir = os.path.join(self.workdir, "task_" + str(i))
+            task.set_workdir(new_tdir, chroot=True)
+
     def __len__(self):
         return len(self._tasks)
 
@@ -340,6 +354,7 @@ class Workflow(BaseWorkflow):
         """True if PAW calculation."""
         return all(task.ispaw for task in self)
 
+    @property
     def status_counter(self):
         """
         Returns a `Counter` object that counts the number of task with 
@@ -363,10 +378,9 @@ class Workflow(BaseWorkflow):
                 `TaskManager` object or None
         """
         for i, task in enumerate(self):
-            #print(hasattr(task, "manager"))
+
             if not hasattr(task, "manager"):
                 # Set the manager
-
                 if manager is not None:
                     # Use the one provided in input.
                     task.set_manager(manager)
@@ -1067,9 +1081,8 @@ class BandStructureWorkflow(Workflow):
         # Register the NSCF run and its dependency.
         self.nscf_task = self.register(nscf_input, deps={self.scf_task: "DEN"}, task_class=NscfTask)
 
-        # Add DOS computation if requested.
+        # Add DOS computation(s) if requested.
         if dos_inputs is not None:
-
             if not isinstance(dos_inputs, (list, tuple)):
                 dos_inputs = [dos_inputs]
 
@@ -1136,7 +1149,8 @@ class DeltaFactorWorkflow(Workflow):
 
     def __init__(self, structure_or_cif, pseudo, kppa,
                  spin_mode="polarized", toldfe=1.e-8, smearing="fermi_dirac:0.1 eV",
-                 accuracy="normal", ecut=None, ecutsm=0.05, chksymbreak=0, workdir=None, manager=None): 
+                 accuracy="normal", ecut=None, pawecutdg=None, ecutsm=0.05, chksymbreak=0,
+                 workdir=None, manager=None, **kwargs):
                  # FIXME Hack in chksymbreak
         """
         Build a `Workflow` for the computation of the deltafactor.
@@ -1187,12 +1201,15 @@ class DeltaFactorWorkflow(Workflow):
             new_structure = AbiStructure.asabistructure(new_structure)
 
             extra_abivars = dict(
+                pawecutdg=pawecutdg,
                 ecutsm=ecutsm,
                 toldfe=toldfe,
                 prtwf=0,
                 paral_kgb=0,
             )
 
+            extra_abivars.update(**kwargs)
+
             if ecut is not None:
                 extra_abivars.update({"ecut": ecut})
 
@@ -1235,7 +1252,8 @@ class DeltaFactorWorkflow(Workflow):
             wien2k = df_database().get_entry(self.pseudo.symbol)
                                                                                                  
             # Compute deltafactor estimator.
-            dfact = df_compute(wien2k.v0, wien2k.b0_GPa, wien2k.b1, eos_fit.v0, eos_fit.b0_GPa, eos_fit.b1, b0_GPa=True)
+            dfact = df_compute(wien2k.v0, wien2k.b0_GPa, wien2k.b1,
+                               eos_fit.v0, eos_fit.b0_GPa, eos_fit.b1, b0_GPa=True)
 
             print("delta",eos_fit)
             print("Deltafactor = %.3f meV" % dfact)
@@ -1301,13 +1319,13 @@ class G0W0_Workflow(Workflow):
         super(G0W0_Workflow, self).__init__(workdir=workdir, manager=manager)
 
         # Register the GS-SCF run.
-        self.scf_task = self.register(scf_input, task_class=ScfTask)
+        self.scf_task = scf_task = self.register(scf_input, task_class=ScfTask)
 
         # Construct the input for the NSCF run.
-        self.nscf_task = self.register(nscf_input, deps={self.scf_task: "DEN"}, task_class=NscfTask)
+        self.nscf_task = nscf_task = self.register(nscf_input, deps={scf_task: "DEN"}, task_class=NscfTask)
 
         # Register the SCREENING run.
-        self.scr_task = self.register(scr_input, deps={self.nscf_task: "WFK"})
+        self.scr_task = scr_task = self.register(scr_input, deps={nscf_task: "WFK"})
 
         # Register the SIGMA runs.
         if not isinstance(sigma_inputs, (list, tuple)): 
@@ -1315,7 +1333,36 @@ class G0W0_Workflow(Workflow):
 
         self.sigma_tasks = []
         for sigma_input in sigma_inputs:
-            self.sigma_tasks.append(self.register(sigma_input, deps={self.nscf_task: "WFK", self.scr_task: "SCR"}))
+            task = self.register(sigma_input, deps={nscf_task: "WFK", scr_task: "SCR"})
+            self.sigma_tasks.append(task)
+
+
+class SigmaConvWorkflow(Workflow):
+
+    def __init__(self, wfk_node, scr_node, sigma_inputs, workdir=None, manager=None):
+        """
+        Workflow for self-energy convergence studies.
+
+        Args:
+            wfk_node:
+                The node who has produced the WFK file
+            scr_node:
+                The node who has produced the SCR file
+            sigma_inputs:
+                List of Strategies for the self-energy run.
+            workdir:
+                Working directory of the calculation.
+            manager:
+                `TaskManager` object.
+        """
+        super(SigmaConvWorkflow, self).__init__(workdir=workdir, manager=manager)
+
+        # Register the SIGMA runs.
+        if not isinstance(sigma_inputs, (list, tuple)): 
+            sigma_inputs = [sigma_inputs]
+
+        for sigma_input in sigma_inputs:
+            self.register(sigma_input, deps={wfk_node: "WFK", scr_node: "SCR"})
 
 
 #class SCGW_Workflow(Workflow):
